==> audiochain-1.1.5-src.tgz: org/audiochain/ui/gui/Dialog.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import static javax.swing.GroupLayout.Alignment.*;
import static javax.swing.LayoutStyle.ComponentPlacement.*;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashSet;

import javax.swing.BorderFactory;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.ParallelGroup;
import javax.swing.GroupLayout.SequentialGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

abstract public class Dialog {
	public static enum YesNoCancel {
		Yes, No, Cancel
	}

	private final Object caller;
	private final JButton enterButton;
	private final JButton escapeButton;
	private final String title;
	private final Collection<JButton> allButtons;
	private JButton resultButton;
	private JDialog dialog;
	private JFrame frame;
	private JCheckBox check;
	private JLabel titleLabel;
	private boolean disposedSuccessfully;
	private boolean hideShowMessageAgainCheckbox;
	private boolean closeManually;
	
	public Dialog(Object caller, String title, final JButton enterAndEscapeButton) {
		this(caller, title, enterAndEscapeButton, enterAndEscapeButton);
	}

	/**
	 * Creates a new Dialog instance.
	 * @param caller the calling parent swing component.
	 * @param title a title used for identifying the message type to offer the show message again checkbox
	 * @param enterButton the button which will be chosen, when the enter key is pressed
	 * @param escapeButton the button which will be chosen, when the escape key is pressed
	 * @param additionalButtons any other buttons. The order of all buttons can also defined by setting this array, even if they are
	 * one of the enter or the escape buttons.
	 */
	public Dialog(Object caller, String title, final JButton enterButton, final JButton escapeButton, JButton... additionalButtons) {
		this.caller = caller;
		this.title = title;
		this.enterButton = enterButton;
		this.escapeButton = escapeButton;
		this.allButtons = new LinkedHashSet<JButton>();
		if (additionalButtons != null) {
			this.allButtons.addAll(Arrays.asList(additionalButtons));
		}
		this.allButtons.add(enterButton);
		this.allButtons.add(escapeButton);
	}

	public static YesNoCancel yesNoCancel(Object caller, String title, String text) {
		JButton yesButton = new JButton("Yes");
		JButton noButton = new JButton("No");
		JButton cancelButton = new JButton("Cancel");
		JButton result = createMessageDialog(caller, title, text, yesButton, cancelButton, yesButton, noButton, cancelButton);
		if (result == yesButton) {
			return YesNoCancel.Yes;
		} else if (result == noButton) {
			return YesNoCancel.No;
		} else if (result == cancelButton) {
			return YesNoCancel.Cancel;
		} else {
			throw new IllegalStateException();
		}
	}

	public static void showTextFromFileDialog(Object caller, String title, String textFileClassPath, int rows, int cols, String closeButtonName) throws IOException {
		InputStream is = caller.getClass().getResourceAsStream(textFileClassPath);
		BufferedReader br = new BufferedReader(new InputStreamReader(is));
		String line;
		StringBuilder sb = new StringBuilder();
		String linesep = System.getProperty("line.separator");
		while ((line = br.readLine()) != null) {
			sb.append(line).append(linesep);
		}
		showTextDialog(caller, title, sb.toString(), rows, cols, closeButtonName);
	}

	public static void showTextDialog(Object caller, String title, final String text, final int rows, final int cols, String closeButtonName) {
		JButton closeButton = new JButton(closeButtonName);
		Dialog dialog = new Dialog(caller, title, closeButton){
			@Override
			protected JComponent createContentComponent() {
				JTextArea area = new JTextArea(rows, cols);
				area.setEditable(false);
				area.setText(text);
				area.setCaretPosition(0);
				return CommonComponentFactory.createScrollPane(area);
			}
		};
		dialog.setHideShowMessageAgainCheckbox(true);
		dialog.open();
	}

	public static void message(Object caller, String title, Throwable e) {
		message(caller, title, e.getMessage());
	}

	public static void message(final Object caller, final String title, final String text) {
		JButton closeButton = new JButton("Close");
		createMessageDialog(caller, title, text, closeButton, closeButton);
	}

	public static void asyncMessage(final Object caller, final String title, final Throwable ex) {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					message(caller, title, ex);
				}
			});
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
	}

	public static void asyncMessage(final Object caller, final String title, final String message) {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					message(caller, title, message);
				}
			});
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
	}

	public static boolean okCancel(Object caller, String title, String text) {
		JButton okButton = new JButton("OK");
		JButton cancelButton = new JButton("Cancel");
		return createMessageDialog(caller, title, text, okButton, cancelButton) == okButton;
	}
	
	private static JButton createMessageDialog(Object caller, String title, final String text, final JButton enterButton, final JButton escapeButton, JButton... buttons) {
		Dialog dialog = new Dialog(caller, title, enterButton, escapeButton, buttons){
			@Override
			protected JComponent createContentComponent() {
				return new JLabel("<html><body style='width:300px;'>" + text + "</body></html>");
			}
		};
		dialog.open();
		return dialog.getResultButton();
	}
	
	public static boolean isButtonPressed(Object caller, String title, final JComponent contentComponent, JButton button) {
		Dialog dialog = new Dialog(caller, title, button, new JButton("Cancel")) {
			@Override
			protected JComponent createContentComponent() {
				return contentComponent;
			}
		};
		dialog.setHideShowMessageAgainCheckbox(true);
		dialog.open();
		return button.equals(dialog.getResultButton());
	}

	abstract protected JComponent createContentComponent();

	private void init() {
		/* Assigns each button an individual (or no, if no distinct chars are left) mnemonic by its label. */
		Collection<Character> givenMnemonics = new HashSet<Character>();
		for (JButton button : allButtons) {
			String text = button.getText();
			char[] charArray = text.toCharArray();
			for (int i = 0, n = charArray.length; i < n; i++) {
				char c = charArray[i];
				if (!givenMnemonics.contains(c)) {
					button.setMnemonic(c);
					givenMnemonics.add(c);
					break;
				}
			}
		}

		frame = findFrame(caller);
		dialog = new JDialog(frame, title, true);
		ComponentMoveMouseAdapter moveAdapter = new ComponentMoveMouseAdapter(dialog);
		dialog.addMouseListener(moveAdapter);
		dialog.addMouseMotionListener(moveAdapter);

		dialog.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentShown(ComponentEvent e) {
				dialog.requestFocus();
			}
		});
		KeyAdapter keyListener = new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				int keyCode = e.getKeyCode();
				switch (keyCode) {
				case 10:
				case 32:
					enterButton.doClick();
					break;
				case 27:
					escapeButton.doClick();
					break;
				}
			}
		};
		dialog.addKeyListener(keyListener);
		dialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				escapeButton.doClick();
			}
		});

		JComponent contentPane = new BackgroundImageComponent("/org/audiochain/ui/gui/background.png");
		contentPane.setBorder(BorderFactory.createEtchedBorder());
		// make the background a bit brighter
		contentPane.setBackground(new Color(contentPane.getBackground().getRGB() + 0x22222222));
		dialog.setContentPane(contentPane);

		titleLabel = new JLabel(title);
		titleLabel.setFocusable(false);

		check = new LightEmittingDiodeCheckBox("Do not show this message again in future.");
		check.setOpaque(false);
		check.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				int keyCode = e.getKeyCode();
				switch (keyCode) {
				case 10:
					enterButton.doClick();
					break;
				case 27:
					escapeButton.doClick();
					break;
				}
			}
		});

		GroupLayout layout = new GroupLayout(contentPane);
		contentPane.setLayout(layout);
		layout.setAutoCreateContainerGaps(true);
		layout.setAutoCreateGaps(true);

		ParallelGroup hGroup = layout.createParallelGroup(CENTER);
		JComponent contentComponent = createContentComponent();
		layout.setHorizontalGroup(hGroup.addComponent(titleLabel, LEADING).addComponent(contentComponent, LEADING));

		SequentialGroup vGroup = layout.createSequentialGroup();
		layout.setVerticalGroup(vGroup.addComponent(titleLabel).addPreferredGap(UNRELATED).addComponent(contentComponent).addPreferredGap(UNRELATED));

		if (!hideShowMessageAgainCheckbox) {
			hGroup.addComponent(check, LEADING);
			vGroup.addComponent(check).addPreferredGap(UNRELATED);
		}

		SequentialGroup hButtonGroup = layout.createSequentialGroup();
		hGroup.addGroup(hButtonGroup);

		ParallelGroup vButtonGroup = layout.createParallelGroup();
		vGroup.addGroup(vButtonGroup);
		
		// Default result when window closing event occurs
		resultButton = escapeButton;

		for (final JButton button : allButtons) {
			hButtonGroup.addComponent(button);
			vButtonGroup.addComponent(button);
			button.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					resultButton = button;
					if (!closeManually && dialog.isVisible()) {
						close();
					}
				}
			});
		}
		layout.linkSize(allButtons.toArray(new JButton[0]));

		dialog.setUndecorated(true);
		dialog.pack();
		dialog.setLocationRelativeTo(frame);
	}

	public void open() {
		JButton defaultResult = findDefaultResult(caller, title);
		if (defaultResult != null) {
			resultButton = defaultResult;
			return;
		}
		if(dialog==null) {
			init();
		}
		// open and wait until dialog closes
		dialog.setVisible(true);
		// called once dialog closes
		if (check.isSelected() && (enterButton == escapeButton || resultButton != escapeButton)) {
			GuiSettings guiSettings = GlobalGuiContext.getGuiSettings();
			guiSettings.getDialogResults().add(createDefaultDialogResult(caller, title, resultButton));
			guiSettings.store();
		}
		disposedSuccessfully = true;
	}
	
	public boolean isOpen() {
		return dialog.isVisible();
	}

	public void close() {
		if (!dialog.isVisible()) {
			return;
		}
		dialog.setVisible(false);
		dialog.dispose();

		/*
		 * check wether this dialog closes regularly, if not, an exception will
		 * be thrown. This procedure is done due to a problem with open jdk: the
		 * dialog blocks on setVisible(true) even if the dialog becomes
		 * disposed. A workaround is to run some dialogs via
		 * SwingUtilities.invokeAndWait() method. There for the static
		 * asyncMessage() methods are provided in this Dialog class. These
		 * methods should be used for Dialogs that will come up asynchronously
		 * for example triggered on listeners within other thread.
		 * 
		 * With sun jdk, everything is fine without this workaround.
		 */
		new Thread() {
			@Override
			public void run() {
				long start = System.currentTimeMillis();
				while (System.currentTimeMillis() - start < 5000) {
					try {
						Thread.sleep(500);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				if (!disposedSuccessfully) {
					throw new IllegalStateException("The dialog '" + title + "' did not dispose regularly. The async message dialog should be used here.");
				}
			}
		}.start();
	}

	private JButton findDefaultResult(Object caller, String title) {
		GuiSettings guiSettings = GlobalGuiContext.getGuiSettings();
		for (JButton button : allButtons) {
			if (guiSettings.getDialogResults().contains(createDefaultDialogResult(caller, title, button))) {
				return button;
			}
		}
		return null;
	}

	private static GuiSettingsDialogResult createDefaultDialogResult(Object caller, String title, JButton button) {
		String resultString = caller.getClass().getSimpleName() + title + button.getText();
		resultString = resultString.trim().replace(' ', '_');
		return new GuiSettingsDialogResult(resultString);
	}

	public static JFrame findFrame(Object parent) {
		if (parent instanceof JFrame) {
			return (JFrame) parent;
		}
		if (parent instanceof Component) {
			return findFrame(((Component) parent).getParent());
		}
		JFrame dummy = new JFrame();
		dummy.setBounds(GlobalGuiContext.getGuiSettings().getWindowBounds());
		return dummy;
	}

	public JButton getResultButton() {
		return resultButton;
	}

	public JDialog getDialog() {
		if(dialog==null) {
			init();
		}
		return dialog;
	}

	public boolean isHideShowMessageAgainCheckbox() {
		return hideShowMessageAgainCheckbox;
	}

	public void setHideShowMessageAgainCheckbox(boolean hideShowMessageAgainCheckbox) {
		this.hideShowMessageAgainCheckbox = hideShowMessageAgainCheckbox;
	}

	public boolean isCloseManually() {
		return closeManually;
	}

	public void setCloseManually(boolean closeManually) {
		this.closeManually = closeManually;
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/ui/gui/Dialog.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import static javax.swing.GroupLayout.Alignment.CENTER;
import static javax.swing.GroupLayout.Alignment.LEADING;
import static javax.swing.LayoutStyle.ComponentPlacement.UNRELATED;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashSet;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.ParallelGroup;
import javax.swing.GroupLayout.SequentialGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

abstract public class Dialog {
	public static enum YesNoCancel {
		Yes, No, Cancel
	}

	private final Object caller;
	private final JButton enterButton;
	private final JButton escapeButton;
	private final String title;
	private final Collection<JButton> allButtons;
	private JButton resultButton;
	private JDialog dialog;
	private JFrame frame;
	private JCheckBox check;
	private JLabel titleLabel;
	private boolean disposedSuccessfully;
	private boolean hideShowMessageAgainCheckbox;
	private boolean closeManually;
	
	public Dialog(Object caller, String title, final JButton enterAndEscapeButton) {
		this(caller, title, enterAndEscapeButton, enterAndEscapeButton);
	}

	/**
	 * Creates a new Dialog instance.
	 * @param caller the calling parent swing component.
	 * @param title a title used for identifying the message type to offer the show message again checkbox
	 * @param enterButton the button which will be chosen, when the enter key is pressed
	 * @param escapeButton the button which will be chosen, when the escape key is pressed
	 * @param additionalButtons any other buttons. The order of all buttons can also defined by setting this array, even if they are
	 * one of the enter or the escape buttons.
	 */
	public Dialog(Object caller, String title, final JButton enterButton, final JButton escapeButton, JButton... additionalButtons) {
		this.caller = caller;
		this.title = title;
		this.enterButton = enterButton;
		this.escapeButton = escapeButton;
		this.allButtons = new LinkedHashSet<JButton>();
		if (additionalButtons != null) {
			this.allButtons.addAll(Arrays.asList(additionalButtons));
		}
		this.allButtons.add(enterButton);
		this.allButtons.add(escapeButton);
	}

	public static YesNoCancel yesNoCancel(Object caller, String title, String text) {
		JButton yesButton = new JButton("Yes");
		JButton noButton = new JButton("No");
		JButton cancelButton = new JButton("Cancel");
		JButton result = createMessageDialog(caller, title, text, yesButton, cancelButton, yesButton, noButton, cancelButton);
		if (result == yesButton) {
			return YesNoCancel.Yes;
		} else if (result == noButton) {
			return YesNoCancel.No;
		} else if (result == cancelButton) {
			return YesNoCancel.Cancel;
		} else {
			throw new IllegalStateException();
		}
	}

	public static void showTextFromFileDialog(Object caller, String title, String textFileClassPath, int rows, int cols, String closeButtonName) throws IOException {
		InputStream is = caller.getClass().getResourceAsStream(textFileClassPath);
		BufferedReader br = new BufferedReader(new InputStreamReader(is));
		String line;
		StringBuilder sb = new StringBuilder();
		String linesep = System.getProperty("line.separator");
		while ((line = br.readLine()) != null) {
			sb.append(line).append(linesep);
		}
		showTextDialog(caller, title, sb.toString(), rows, cols, closeButtonName);
	}

	public static void showTextDialog(Object caller, String title, final String text, final int rows, final int cols, String closeButtonName) {
		JButton closeButton = new JButton(closeButtonName);
		Dialog dialog = new Dialog(caller, title, closeButton){
			@Override
			protected JComponent createContentComponent() {
				JTextArea area = new JTextArea(rows, cols);
				area.setEditable(false);
				area.setText(text);
				area.setCaretPosition(0);
				return CommonComponentFactory.createScrollPane(area);
			}
		};
		dialog.setHideShowMessageAgainCheckbox(true);
		dialog.open();
	}

	public static void message(Object caller, String title, Throwable e) {
		message(caller, title, e.getMessage());
	}

	public static void message(final Object caller, final String title, final String text) {
		JButton closeButton = new JButton("Close");
		createMessageDialog(caller, title, text, closeButton, closeButton);
	}

	public static void asyncMessage(final Object caller, final String title, final Throwable ex) {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					message(caller, title, ex);
				}
			});
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
	}

	public static void asyncMessage(final Object caller, final String title, final String message) {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					message(caller, title, message);
				}
			});
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
	}

	public static boolean okCancel(Object caller, String title, String text) {
		JButton okButton = new JButton("OK");
		JButton cancelButton = new JButton("Cancel");
		return createMessageDialog(caller, title, text, okButton, cancelButton) == okButton;
	}
	
	private static JButton createMessageDialog(Object caller, String title, final String text, final JButton enterButton, final JButton escapeButton, JButton... buttons) {
		Dialog dialog = new Dialog(caller, title, enterButton, escapeButton, buttons){
			@Override
			protected JComponent createContentComponent() {
				return new JLabel("<html><body style='width:300px;'>" + text + "</body></html>");
			}
		};
		dialog.open();
		return dialog.getResultButton();
	}
	
	public static boolean isButtonPressed(Object caller, String title, final JComponent contentComponent, JButton button) {
		Dialog dialog = new Dialog(caller, title, button, new JButton("Cancel")) {
			@Override
			protected JComponent createContentComponent() {
				return contentComponent;
			}
		};
		dialog.setHideShowMessageAgainCheckbox(true);
		dialog.open();
		return button.equals(dialog.getResultButton());
	}

	abstract protected JComponent createContentComponent();

	private void init() {
		/* Assigns each button an individual (or no, if no distinct chars are left) mnemonic by its label. */
		Collection<Character> givenMnemonics = new HashSet<Character>();
		for (JButton button : allButtons) {
			String text = button.getText();
			char[] charArray = text.toCharArray();
			for (int i = 0, n = charArray.length; i < n; i++) {
				char c = charArray[i];
				if (!givenMnemonics.contains(c)) {
					button.setMnemonic(c);
					givenMnemonics.add(c);
					break;
				}
			}
		}

		frame = findFrame(caller);
		dialog = new JDialog(frame, title, true);
		ComponentMoveMouseAdapter moveAdapter = new ComponentMoveMouseAdapter(dialog);
		dialog.addMouseListener(moveAdapter);
		dialog.addMouseMotionListener(moveAdapter);

		dialog.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentShown(ComponentEvent e) {
				dialog.requestFocus();
			}
		});
		KeyAdapter keyListener = new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				int keyCode = e.getKeyCode();
				switch (keyCode) {
				case 10:
				case 32:
					enterButton.doClick();
					break;
				case 27:
					escapeButton.doClick();
					break;
				}
			}
		};
		dialog.addKeyListener(keyListener);
		dialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				escapeButton.doClick();
			}
		});

		JComponent contentPane = CommonComponentFactory.createDialogBackgroundComponent();
		dialog.setContentPane(contentPane);

		titleLabel = new JLabel(title);
		titleLabel.setFocusable(false);

		check = new LightEmittingDiodeCheckBox("Do not show this message again in future.");
		check.setOpaque(false);
		check.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				int keyCode = e.getKeyCode();
				switch (keyCode) {
				case 10:
					enterButton.doClick();
					break;
				case 27:
					escapeButton.doClick();
					break;
				}
			}
		});

		GroupLayout layout = new GroupLayout(contentPane);
		contentPane.setLayout(layout);
		layout.setAutoCreateContainerGaps(true);
		layout.setAutoCreateGaps(true);

		ParallelGroup hGroup = layout.createParallelGroup(CENTER);
		JComponent contentComponent = createContentComponent();
		layout.setHorizontalGroup(hGroup.addComponent(titleLabel, LEADING).addComponent(contentComponent, LEADING));

		SequentialGroup vGroup = layout.createSequentialGroup();
		layout.setVerticalGroup(vGroup.addComponent(titleLabel).addPreferredGap(UNRELATED).addComponent(contentComponent).addPreferredGap(UNRELATED));

		if (!hideShowMessageAgainCheckbox) {
			hGroup.addComponent(check, LEADING);
			vGroup.addComponent(check).addPreferredGap(UNRELATED);
		}

		SequentialGroup hButtonGroup = layout.createSequentialGroup();
		hGroup.addGroup(hButtonGroup);

		ParallelGroup vButtonGroup = layout.createParallelGroup();
		vGroup.addGroup(vButtonGroup);
		
		// Default result when window closing event occurs
		resultButton = escapeButton;

		for (final JButton button : allButtons) {
			hButtonGroup.addComponent(button);
			vButtonGroup.addComponent(button);
			button.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					resultButton = button;
					if (!closeManually && dialog.isVisible()) {
						close();
					}
				}
			});
		}
		layout.linkSize(allButtons.toArray(new JButton[0]));

		dialog.setUndecorated(true);
		dialog.pack();
		dialog.setLocationRelativeTo(frame);
	}

	public void open() {
		JButton defaultResult = findDefaultResult(caller, title);
		if (defaultResult != null) {
			resultButton = defaultResult;
			return;
		}
		if(dialog==null) {
			init();
		}
		// open and wait until dialog closes
		dialog.setVisible(true);
		// called once dialog closes
		if (check.isSelected() && (enterButton == escapeButton || resultButton != escapeButton)) {
			GuiSettings guiSettings = GuiSettings.getInstance();
			guiSettings.getDialogResults().add(createDefaultDialogResult(caller, title, resultButton));
			guiSettings.store();
		}
		disposedSuccessfully = true;
	}
	
	public boolean isOpen() {
		return dialog.isVisible();
	}

	public void close() {
		if (!dialog.isVisible()) {
			return;
		}
		dialog.setVisible(false);
		dialog.dispose();

		/*
		 * check wether this dialog closes regularly, if not, an exception will
		 * be thrown. This procedure is done due to a problem with open jdk: the
		 * dialog blocks on setVisible(true) even if the dialog becomes
		 * disposed. A workaround is to run some dialogs via
		 * SwingUtilities.invokeAndWait() method. There for the static
		 * asyncMessage() methods are provided in this Dialog class. These
		 * methods should be used for Dialogs that will come up asynchronously
		 * for example triggered on listeners within other thread.
		 * 
		 * With sun jdk, everything is fine without this workaround.
		 */
		new Thread() {
			@Override
			public void run() {
				long start = System.currentTimeMillis();
				while (System.currentTimeMillis() - start < 5000) {
					try {
						Thread.sleep(500);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				if (!disposedSuccessfully) {
					throw new IllegalStateException("The dialog '" + title + "' did not dispose regularly. The async message dialog should be used here.");
				}
			}
		}.start();
	}

	private JButton findDefaultResult(Object caller, String title) {
		GuiSettings guiSettings = GuiSettings.getInstance();
		for (JButton button : allButtons) {
			if (guiSettings.getDialogResults().contains(createDefaultDialogResult(caller, title, button))) {
				return button;
			}
		}
		return null;
	}

	private static GuiSettingsDialogResult createDefaultDialogResult(Object caller, String title, JButton button) {
		String resultString = caller.getClass().getSimpleName() + title + button.getText();
		resultString = resultString.trim().replace(' ', '_');
		return new GuiSettingsDialogResult(resultString);
	}

	public static JFrame findFrame(Object parent) {
		if (parent instanceof JFrame) {
			return (JFrame) parent;
		}
		if (parent instanceof Component) {
			return findFrame(((Component) parent).getParent());
		}
		JFrame dummy = new JFrame();
		dummy.setBounds(GuiSettings.getInstance().getWindowBounds());
		return dummy;
	}

	public JButton getResultButton() {
		return resultButton;
	}

	public JDialog getDialog() {
		if(dialog==null) {
			init();
		}
		return dialog;
	}

	public boolean isHideShowMessageAgainCheckbox() {
		return hideShowMessageAgainCheckbox;
	}

	public void setHideShowMessageAgainCheckbox(boolean hideShowMessageAgainCheckbox) {
		this.hideShowMessageAgainCheckbox = hideShowMessageAgainCheckbox;
	}

	public boolean isCloseManually() {
		return closeManually;
	}

	public void setCloseManually(boolean closeManually) {
		this.closeManually = closeManually;
	}
}
